package vostokjsonlibsample;

import java.util.ArrayList;
import java.util.List;
import org.vostok.json.JSONContext;
import org.vostok.json.Marshaller;
import vostokjsonlibsample.bean.LatLng;
import vostokjsonlibsample.bean.LatLngConverter;
import vostokjsonlibsample.bean.SimpleBean;

/**
 *
 * @author levreau.jerome@vostoksystem.eu
 */
public class TestBuilder {

	public void buildSomePrimitive() throws Exception {
		Marshaller bp = JSONContext.getInstance().getMarshaller();
		System.out.println("\n\n---------------- building some primitive based ------------------");

		String str = "My string xyz";
		System.out.println("#building a string : " + str + "....");
		System.out.println("answer : " + bp.marshal(str) + " (" + str.getClass().getCanonicalName() + ")\n");

		Double d = Math.PI;
		System.out.println("#building a double : " + d + "....");
		System.out.println("answer : " + bp.marshal(d) + " (" + d.getClass().getCanonicalName() + ")\n");

		Integer[] vala = new Integer[]{10, 20, 30, 40};
		System.out.println("#parsing an Integer array : \"" + vala + "\"....");
		System.out.println("answer : " + bp.marshal(vala) + " (" + vala.getClass().getCanonicalName() + ")\n");



	}

	public void buildSimpleList() throws Exception {

		Marshaller bp = JSONContext.getInstance().getMarshaller();

		List<String> data = new ArrayList<String>();
		data.add("first");
		data.add("second");
		data.add("third");

		System.out.println("\n\n---------------- building a List with generic ------------------");
		System.out.println("#source : " + data + "(" + data.getClass().getCanonicalName() + ") ....");
		System.out.println("#answer : " + bp.marshal(data));
	}

	public void buildSimpleBean() throws Exception {

		Marshaller bp = JSONContext.getInstance().getMarshaller();

		SimpleBean bean = new SimpleBean();
		bean.setVal("my value");

		System.out.println("\n\n---------------- building a Simple bean ------------------");
		System.out.println("#source : " + bean.toString() + "(" + bean.getClass().getCanonicalName() + ") ....");
		System.out.println("#nota : @discard annotation on dummy field");
		System.out.println("#answer : " + bp.marshal(bean));
	}

	public void buildLatLngBean() throws Exception {

		Marshaller bp = JSONContext.getInstance().getMarshaller();
		JSONContext context = new JSONContext(true, LatLng.class.getCanonicalName(), LatLngConverter.class);

		System.out.println("\n\n---------------- building a LatLng object ------------------");
		System.out.println("Demonstrate how to translate a LatLng object not into a js object but an array using converter\n");

		LatLng ll = new LatLng();
		ll.setLat(55.754397);
		ll.setLng(37.619997);

		System.out.println("#source : " + ll + "(" + ll.getClass().getCanonicalName() + ") ....");
		System.out.println("#answer with default builder : " + bp.marshal(ll));
		System.out.println("#answer with extended builder : " + context.getMarshaller().marshal(ll));
	}

	/**
	 * @param args the command line arguments
	 */
	public static void main(String[] args) throws Exception {

		TestBuilder test = new TestBuilder();

		test.buildSomePrimitive();
		test.buildSimpleList();
		test.buildSimpleBean();
		test.buildLatLngBean();

	}
}
