package vostokjsonlibsample;

import java.util.Currency;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.vostok.json.JSONContext;
import org.vostok.json.Unmarshaller;
import vostokjsonlibsample.bean.FirstSubBean;
import vostokjsonlibsample.bean.LatLngConverter;
import vostokjsonlibsample.bean.SecondBean;
import vostokjsonlibsample.bean.SimpleBean;
import vostokjsonlibsample.bean.ThirdBean;

/**
 * A sample showing seval use of the library
 * @author levreau.jerome@vostoksystem.eu
 */
public class TestParser {

	public void parseSomePrimitive() throws Exception {
		Unmarshaller bp = JSONContext.getInstance().getUnmarshaller();

		System.out.println("\n\n---------------- parsing some primitive based ------------------");

		System.out.println("#parsing a string : \"my json string\"....");
		String str = (String) bp.unmarshal(String.class, "my json string");
		System.out.println("answer : " + str + " (" + str.getClass().getCanonicalName() + ")\n");

		System.out.println("#parsing an integer :\"666\"....");
		Integer val = (Integer) bp.unmarshal(Integer.class, "666");
		System.out.println("answer : " + val + " (" + val.getClass().getCanonicalName() + ")\n");

		System.out.println("#parsing an int primitive :\"666\"....");
		int val2 = (Integer) bp.unmarshal(int.class, "666");
		System.out.println("answer : " + val2 + " " + "\n");

		System.out.println("#parsing a simple int array : \"[1,2,3,4]\"....");
		int[] vali = (int[]) bp.unmarshal(int[].class, "[1,2,3,4]");
		System.out.println("answer : " + vali[0] + " " + vali[1] + " " + vali[2] + " " + vali[3] + " (" + vali.getClass().getCanonicalName() + ")\n");

		System.out.println("#parsing an Integer array : \"[10,20,30,40]\"....");
		Integer[] vala = (Integer[]) bp.unmarshal(Integer[].class, "[10,20,30,40]");
		System.out.println("answer : " + vala[0] + " " + vala[1] + " " + vala[2] + " " + vala[3] + " (" + vala.getClass().getCanonicalName() + ")\n");
	}

	public void parseGeneric() throws Exception {
		Unmarshaller bp = JSONContext.getInstance().getUnmarshaller();
		String data = "[\"first\",\"second\",\"third\"]";

		System.out.println("\n\n---------------- parsing List with generic ------------------");
		System.out.println("#source : " + data + " ....");

		List<String> answer = (List<String>) bp.unmarshal(List.class, String.class, data);
		System.out.println("Start loop, list is " + answer.size() + " element(s) (" + answer.getClass().getCanonicalName() + ")");
		for (String s : answer) {
			System.out.println("answer from List : " + s);
		}
	}

	public void parseEmbedList() throws Exception {

		Unmarshaller bp = JSONContext.getInstance().getUnmarshaller();

		String data = "[[\"c0-0\", \"c0-1\"],[\"c1-0\"],[\"c2-0\", \"c2-1\"]]";

		System.out.println("\n\n---------------- parsing some linked list  ------------------");
		System.out.println("format : <List<List<String>>");
		System.out.println("#source : " + data + " ....");

		List<List<String>> list = (List<List<String>>) bp.unmarshal(List.class, List.class, data);
		System.out.println("number of element in the list : " + list.size() + " (" + list.getClass().getCanonicalName() + ")");

		int i = 0;
		for (List<String> l : list) {
			System.out.println("level 1, index " + (i++) + ", " + l.size() + " child(s)");
			for (String s : l) {
				System.out.println("     " + s);
			}
		}
	}

	public void parseArrayOfBean() throws Exception {

		Unmarshaller bp = JSONContext.getInstance().getUnmarshaller();
		String data = "[{val:\"first\"},{val:\"second\"},{val:\"third\"}]";

		System.out.println("\n\n---------------- parsing a simple array of bean ------------------");
		System.out.println("#source : " + data + " ....");

		SimpleBean[] array = (SimpleBean[]) bp.unmarshal(SimpleBean[].class, data);

		System.out.println("Start loop, array is " + array.length + " element(s) (" + array.getClass().getCanonicalName() + ")");
		for (SimpleBean b : array) {
			System.out.println("answer from bean : " + b.getVal());
		}
	}

	public void parseLatLngBean() throws Exception {
		Map m = new HashMap();

		m.put(LatLngConverter.class.getCanonicalName(), LatLngConverter.class);

		JSONContext context = new JSONContext(true, m);
		String data = "  [ 55.754397,   37.619997]";

		System.out.println("\n\n---------------- parsing an array into a LatLng bean ------------------");
		System.out.println("#source : " + data + " ....");
		System.out.println("answer  : " + context.getUnmarshaller().unmarshal(LatLngConverter.class, data));
	}

	public void parseFullBean() throws Exception {

		Unmarshaller bp = JSONContext.getInstance().getUnmarshaller();
		String data = "{aDouble:10.25, anInteger:666, "
			+ "simpleStringArray:[\"a\", \"b\",\"c\"] ,"
			+ "aSecondBean:{aString:\"azerty\",simpleStringList:[\"a0-0\",\"a0-1\"]} , "
			+ "simpleStringList:[\"abcd\"],"
			+ "complexIntegeArray:[[1,2,3, 4],[10,20,30]], "
			+ "listOfSecondBean:[ {aString:\"first sbl\",simpleStringList:[\"b0-0\",\"b0-1\",\"b0-2\"]},"
			+ " {aString:\"second sbl\",simpleStringList:[\"b1-0\"]}  ]  }";

		System.out.println("\n\n---------------- parsing complex bean ------------------");
		System.out.println("#source : " + data);

		FirstSubBean bean = (FirstSubBean) bp.unmarshal(FirstSubBean.class, data);
		System.out.println("----------------------------------");

		System.out.println("value of anInteger : " + bean.getAnInteger());
		System.out.println("value of aDouble : " + bean.getaDouble());

		System.out.println("\nvalue of simpleStringArray, length " + bean.getSimpleStringArray().length);
		for (String s : bean.getSimpleStringArray()) {
			System.out.println("--> simpleStringArray : " + s);
		}

		System.out.println("\nIs there a SecondBean ?? " + (bean.getaSecondBean() != null));
		if (bean.getaSecondBean() != null) {
			SecondBean sb = bean.getaSecondBean();
			System.out.println("aSecondBean.aString : " + sb.getaString());
			System.out.println("data for aSecondBean.simpleStringList...");
			for (String s : sb.getSimpleStringList()) {
				System.out.println("--> aSecondBean.simpleStringList : " + s);
			}
		}

		System.out.println("\nIs there a simpleStringList ?? " + (bean.getSimpleStringList() != null));
		if (bean.getSimpleStringList() != null) {
			System.out.println("simpleStringList : " + bean.getSimpleStringList().size() + " element(s)");
			System.out.println("data for simpleStringList...");
			for (String s : bean.getSimpleStringList()) {
				System.out.println("--> simpleStringList : " + s);
			}
		}

		System.out.println("\nIs there a complexIntegeArray ?? " + (bean.getComplexIntegeArray() != null));
		if (bean.getComplexIntegeArray() != null) {
			System.out.println("complexIntegeArray : " + bean.getComplexIntegeArray().length + " element(s)");
			for (List<Integer> l : bean.getComplexIntegeArray()) {
				System.out.println("new list....");

				for (Integer i : l) {
					System.out.println("value : " + i);
				}
			}
			System.out.println("...end of complexIntegeArray");
		}

		System.out.println("\nIs there a list of bean ?? " + (bean.getListOfSecondBean() != null));

		if (bean.getListOfSecondBean() != null) {
			System.out.println("list of SecondBean bean, " + bean.getListOfSecondBean().size() + " element(s)");

			for (SecondBean sb : bean.getListOfSecondBean()) {
				System.out.println(", new SecondBean bean");
				System.out.println("SecondBean.aString : " + sb.getaString());
				System.out.println("data for SecondBean.simpleStringList...");
				for (String s : sb.getSimpleStringList()) {
					System.out.println("--> SecondBean.simpleStringList : " + s);
				}
			}

			System.out.println("... end of Second bean list");
		}
	}

	public void parseComplexList() throws Exception {
// TODO : NOT YET UIMPLEMENTED
		Unmarshaller bp = JSONContext.getInstance().getUnmarshaller();
		String data = "["
			+ "{array:[[\"c0-0\", \"c0-1\"],[\"c1-0\"],[\"c2-0\", \"c2-1\"]]}"
			+ "]";

		System.out.println("\n\n---------------- parsing some complex list  ------------------");
		System.out.println("format : List<ThirdBean>, ThirdBean.array=List<List<String>>");
		System.out.println("#source : " + data + " ....");

		System.out.println("unfortunately : NOT YET UIMPLEMENTED !");

		try {
			List<ThirdBean> list = (List<ThirdBean>) bp.unmarshal(List.class, ThirdBean.class, data);
			System.out.println("number of bean in the list : " + list.size());

			for (ThirdBean b : list) {
				System.out.println("#new bean ,list all value(s), recursivelly");

				for (List<String> l : b.getArray()) {
					for (String s : l) {
						System.out.println("value : " + s);
					}
				}

				System.out.println("... end of value");
			}
		} catch (Exception e) {
			System.out.println("yeah, throw an exeption.... ( " + e.getClass().getCanonicalName() + " )");
		}
	}

	public void parseCurency() throws Exception {

		String data = "EUR";
		Unmarshaller bp = JSONContext.getInstance().getUnmarshaller();

		System.out.println("\n\n---------------- parsing a currency code  ------------------");
		System.out.println("#source : " + data + " ....");

		Currency set = (Currency) bp.unmarshal(Currency.class, null, data);

		System.out.println("answer : code is " + set.getCurrencyCode() + ", name is " + set.getDisplayName());
	}

	/**
	 * @param args the command line arguments
	 */
	public static void main(String[] args) throws Exception {

		System.out.println("this sample will perform some parsing on json formated source string, from simple primitive to complex structure.\n\n");

		TestParser test = new TestParser();

		test.parseSomePrimitive();
		test.parseCurency();
		test.parseGeneric();
		test.parseLatLngBean();
		test.parseArrayOfBean();
		test.parseEmbedList();
		test.parseFullBean();
		test.parseComplexList();
	}
}
